package {
	import flash.events.Event;
	
	import mx.core.UIComponent;
	import mx.events.ResizeEvent;
	
	import org.papervision3d.cameras.Camera3D;
	import org.papervision3d.materials.BitmapFileMaterial;
	import org.papervision3d.objects.primitives.Plane;
	import org.papervision3d.render.BasicRenderEngine;
	import org.papervision3d.scenes.Scene3D;
	import org.papervision3d.view.Viewport3D;

	public class SkyboxComponent extends UIComponent{
		public var boxWidth:int=512;
		public var boxHeight:int=512;
		
		public var front:String="images/front.jpg";
		public var back:String="images/back.jpg";
		public var left:String="images/left.jpg";
		public var right:String="images/right.jpg";
		public var top:String="images/top.jpg";
		public var bottom:String="images/bottom.jpg";
		
		private var viewport:Viewport3D;
		private var scene:Scene3D;
		private var camera:Camera3D;
		private var renderer:BasicRenderEngine;
		
		public function SkyboxComponent(){
			super();
			//init on 'added_to_stage' event so width and height are set properly
			this.addEventListener(Event.ADDED_TO_STAGE,init);
			//resize viewport when component resizes
			this.addEventListener(ResizeEvent.RESIZE,onResize);
		}
		
		private function init(event:Event):void{
			//setup papervision
			viewport=new Viewport3D(this.width,this.height);
			renderer=new BasicRenderEngine();
			scene=new Scene3D();
			camera=new Camera3D(60,10,Number.POSITIVE_INFINITY,true);
			camera.zoom=10;
			camera.focus=Math.round(((boxWidth+boxHeight)/2)/20);
			camera.x=0;
			camera.y=0;
			camera.z=0;
			this.addChild(viewport);
			//here we will place the images, by using planes instead of a cube we can ensure correct image orientation
			//front plane
			var mat:BitmapFileMaterial=new BitmapFileMaterial(front);
			var plane:Plane=new Plane(mat,boxWidth,boxHeight,8,8);
			plane.z=boxWidth/2;
			scene.addChild(plane);
			//back plane
			mat=new BitmapFileMaterial(back);
			plane=new Plane(mat,boxWidth,boxHeight,8,8);
			plane.z=-(boxWidth/2);
			plane.rotationY=180;
			scene.addChild(plane);
			//left plane
			mat=new BitmapFileMaterial(left);
			plane=new Plane(mat,boxWidth,boxHeight,8,8);
			plane.x=-(boxWidth/2);
			plane.rotationY=-90;
			scene.addChild(plane);
			//right plane
			mat=new BitmapFileMaterial(right);
			plane=new Plane(mat,boxWidth,boxHeight,8,8);
			plane.x=boxWidth/2;
			plane.rotationY=90;
			scene.addChild(plane);
			//top plane
			mat=new BitmapFileMaterial(top);
			plane=new Plane(mat,boxWidth,boxHeight,8,8);
			plane.y=boxHeight/2;
			plane.rotationX=-90;
			scene.addChild(plane);
			//bottom plane
			mat=new BitmapFileMaterial(bottom);
			plane=new Plane(mat,boxWidth,boxHeight,8,8);
			plane.y=-(boxHeight/2);
			plane.rotationX=90;
			scene.addChild(plane);
			//render on 'enter_frame' event
			this.addEventListener(Event.ENTER_FRAME,onEnterFrame);
		}
		
		//the camera will always stay in the center, only its rotation should be changed
		public function setCameraRotation(x:Number,y:Number,z:Number):void{
			camera.rotationX=x;
			camera.rotationY=y;
			camera.rotationZ=z;
		}
		
		private function onEnterFrame(event:Event):void{
			//render
			renderer.renderScene(scene,camera,viewport);
		}
		
		private function onResize(event:ResizeEvent):void{
			if(viewport!=null){
				//resize viewport
				viewport.viewportWidth=this.width;
				viewport.viewportHeight=this.height
			}
		}
	}
}